#!/usr/bin/python
import unittest

from . import create_cvss_format
from . import parse_cvss_format


class TestCVSSFormatter(unittest.TestCase):

    def test_format_critical_cvss_v2(self):
        """ tests creating a jira table comment for a critical cvss v2 """
        vector_string = 'AV:N/AC:L/Au:N/C:P/I:P/A:C'
        expected = """
Proposed CVSS score: 9.0 => *Critical* severity

*Exploitability Metrics*

|| AccessVector | Network |
|| AccessComplexity | Low |
|| Authentication | None |


*Impact Metrics*

|| ConfImpact | Partial |
|| IntegImpact | Partial |
|| AvailImpact | Complete |

"""
        formatted_string = create_cvss_format.cvss_v2_vector_to_jira_table(
            vector_string,
            9.0,
        )
        self.assertEqual(
            formatted_string,
            expected,
        )

        parsed = parse_cvss_format.parse_text_info_score(formatted_string)
        self.assertEqual(parsed['version'], 2.0)
        self.assertEqual(parsed['score'], 9.0)
        self.assertEqual(parsed['exploitability_sub_score'], 10.0)
        self.assertEqual(parsed['AccessVector'], 'Network')
        self.assertEqual(parsed['AccessComplexity'], 'Low')
        self.assertEqual(parsed['Authentication'], 'None')
        for key in ['ConfImpact', 'IntegImpact']:
            self.assertEqual(parsed[key], 'Partial')
        self.assertEqual(parsed['AvailImpact'], 'Complete')

    def test_format_critical_cvss_v3(self):
        """ tests creating a jira table comment for a critical cvss v3 """
        vector_string = 'AV:N/AC:L/PR:N/UI:R/S:C/C:H/I:H/A:H'
        expected = """
Proposed CVSS v3 score: 9.6 => *Critical* severity

*Exploitability Metrics*

|| Attack Vector | Network |
|| Attack Complexity | Low |
|| Privileges Required | None |
|| User Interaction | Required |

*Scope Metric*

|| Scope | Changed |

*Impact Metrics*

|| Confidentiality | High |
|| Integrity | High |
|| Availability | High |

"""
        formatted_string = create_cvss_format.cvss_v3_vector_to_jira_table(
            vector_string,
            9.6,
        )
        self.assertEqual(
            formatted_string,
            expected,
        )

        parsed = parse_cvss_format.parse_text_info_score(formatted_string)
        self.assertEqual(parsed['version'], 3.0)
        self.assertEqual(parsed['score'], 9.6)
        self.assertEqual(parsed['exploitability_sub_score'], 2.84)
        self.assertEqual(parsed['Attack Vector'], 'Network')
        self.assertEqual(parsed['Attack Complexity'], 'Low')
        self.assertEqual(parsed['Privileges Required'], 'None')
        self.assertEqual(parsed['User Interaction'], 'Required')
        for key in ['Confidentiality', 'Integrity', 'Availability']:
            self.assertEqual(parsed[key], 'High')


class TestCVSSParser(unittest.TestCase):

    @property
    def v2_text(self):
        return"""
Proposed CVSS score: 10 => *Critical* severity

*Exploitability Metrics*

|| AccessVector | Network |
|| AccessComplexity | Low |
|| Authentication | None |


*Impact Metrics*

|| ConfImpact | Complete |
|| IntegImpact | Complete |
|| AvailImpact | Complete |

"""

    @property
    def v3_text_example_4_4(self):
        return """
Proposed CVSS v3 score: 9.9 => *Critical* severity

*Exploitability Metrics*

|| Attack Vector | Network |
|| Attack Complexity | Low |
|| Privileges Required | Low |
|| User Interaction | None |

*Scope Metric*

|| Scope | Changed |

*Impact Metrics*

|| Confidentiality | High |
|| Integrity | High |
|| Availability | High |

"""

    def test_parse_critical_cvss_v2(self):
        """ tests parsing a critical cvss v2 score """
        parsed = parse_cvss_format.parse_text_info_score(self.v2_text)
        self.assertEqual(parsed['version'], 2.0)
        self.assertEqual(parsed['score'], 10)
        self.assertEqual(parsed['exploitability_sub_score'], 10.0)
        self.assertEqual(parsed['AccessVector'], 'Network')
        self.assertEqual(parsed['AccessComplexity'], 'Low')
        self.assertEqual(parsed['Authentication'], 'None')
        for key in ['ConfImpact', 'IntegImpact', 'AvailImpact']:
            self.assertEqual(parsed[key], 'Complete')

    def test_parse_cvss_v3_example_4_4(self):
        """ tests parsing & calculating the cvss v3 score of example 4.4
            (base score 9.9) from https://www.first.org/cvss/examples.
        """
        parsed = parse_cvss_format.parse_text_info_score(
            self.v3_text_example_4_4)
        self.assertEqual(parsed['version'], 3.0)
        self.assertEqual(parsed['score'], 9.9)
        self.assertEqual(parsed['exploitability_sub_score'], 3.11)
        self.assertEqual(parsed['Attack Vector'], 'Network')
        self.assertEqual(parsed['Attack Complexity'], 'Low')
        self.assertEqual(parsed['Privileges Required'], 'Low')
        for key in ['Confidentiality', 'Integrity', 'Availability']:
            self.assertEqual(parsed[key], 'High')

    def test_parse_cvss_v3_example_4_4_with_spacing(self):
        """ tests that the parsing of cvss score information
            works regardless of superfluous spacing.
        """
        text = self.v3_text_example_4_4.replace('\n', '\n ')
        parsed = parse_cvss_format.parse_text_info_score(
            text)
        self.assertEqual(parsed['version'], 3.0)
        self.assertEqual(parsed['score'], 9.9)


if __name__ == "__main__":
    unittest.main()



from __future__ import division
import collections
import math


def get_severity_description(score):
    """ Returns the matching qualitative severity rating for the given
        cvss v3 score.
    """
    for severity, range in get_severity_description_rating_dict().items():
        if range.bottom <= score <= range.top:
            return severity
    raise ValueError('Invalid cvss score value')


def get_severity_description_rating_dict():
    """ Returns the qualitative severity rating ranges for
        cvss v3 in a dictionary.
    """
    Range = collections.namedtuple('Range', ['bottom', 'top'])
    return collections.OrderedDict([
        ('None', Range(0.0, 0.0)),
        ('Low', Range(0.1, 3.9)),
        ('Medium', Range(4, 6.9)),
        ('High', Range(7.0, 8.9)),
        ('Critical', Range(9.0, 10.0)),
    ])


def get_scope_value(scope_changed):
    if isinstance(scope_changed, bool):
        return scope_changed
    scope_changed = scope_changed.lower()
    if scope_changed == 'changed':
        return True
    if scope_changed == 'unchanged':
        return False
    raise ValueError('Invalid value for scope changed (%s)' % scope_changed)


def get_attack_vector_value(text):
    val_map = {
        'network': 0.85,
        'adjacent network': 0.62,
        'adjacent': 0.62,
        'local': 0.55,
        'physical': 0.2,
    }
    return val_map[text.lower()]


def get_attack_complexity_value(text):
    val_map = {
        'low': 0.77,
        'high': 0.44,
    }
    return val_map[text.lower()]


def get_privileges_required_value(text, scope_changed):
    val_map = {
        'none': 0.85,
        'low': 0.62,
        'high': 0.27
    }
    scope_changed_val_map = {
        'low': 0.68,
        'high': 0.50,
    }
    if get_scope_value(scope_changed):
        val_map.update(scope_changed_val_map)
    return val_map[text.lower()]


def get_user_interaction_value(text):
    val_map = {
        'none': 0.85,
        'required': 0.62,
    }
    return val_map[text.lower()]


def get_impact_val_map():
    return {
        'high': 0.56,
        'low': 0.22,
        'none': 0
    }


def get_impact_metric_score(text):
    val_map = get_impact_val_map()
    return val_map[text.lower()]


def get_impact_sub_score_base(confidentiality, integrity, availability):
    c = get_impact_metric_score(confidentiality)
    i = get_impact_metric_score(integrity)
    a = get_impact_metric_score(availability)
    return 1 - ((1 - c) * (1 - i) * (1 - a))


def get_impact_sub_score(isc_base, scope_modified):
    if not scope_modified:
        return 6.42 * isc_base
    return (7.52 * (isc_base - 0.029)) - (3.25 * ((isc_base - 0.02) ** 15))


def _get_exploitability_sub_score(av_value, ac_value, pr_value, ui_value):
    return 8.22 * av_value * ac_value * pr_value * ui_value


def get_exploitability_sub_score(
        attack_vector,
        attack_complexity,
        privileges_required,
        user_interaction,
        scope_changed):
    return _get_exploitability_sub_score(
        get_attack_vector_value(attack_vector),
        get_attack_complexity_value(attack_complexity),
        get_privileges_required_value(privileges_required, scope_changed),
        get_user_interaction_value(user_interaction),
    )


def compute_base_score(impact_sub_score, exploit_sub_score, scope_changed):
    if impact_sub_score <= 0:
        return 0
    if not get_scope_value(scope_changed):
        value = min((impact_sub_score + exploit_sub_score), 10)
    else:
        value = min((1.08 * (impact_sub_score + exploit_sub_score)), 10)
    value = math.ceil(value * 10) / 10
    return value


def compute_base_score_from_dicts(
        exploitability_dict, impact_dict, scope_changed):
    """ returns the base score calculated from the given
        exploitability dictionary, impact dictionary and scope changed
        arguments.
    """
    scope_changed = get_scope_value(scope_changed)
    exploit_args = exploitability_dict.copy()
    exploit_args['scope_changed'] = scope_changed
    exploitability_sub_score = get_exploitability_sub_score(
        **exploit_args)
    isc_base = get_impact_sub_score_base(**impact_dict)
    impact_sub_score = get_impact_sub_score(
        isc_base,
        scope_changed,
    )
    return compute_base_score(
        impact_sub_score, exploitability_sub_score, scope_changed)


def get_base_vector_string_dict():
    """ returns the base vector string dictionary. """
    impact_dict = {
        'N': 'None',
        'L': 'Low',
        'H': 'High',
    }
    return {
        'av': {
            'N': 'Network',
            'A': 'Adjacent',
            'L': 'Local',
            'P': 'Physical',
        },
        'ac': {
            'L': 'Low',
            'H': 'High',
        },
        'pr': {
            'N': 'None',
            'L': 'Low',
            'H': 'High',
        },
        'ui': {
            'N': 'None',
            'R': 'Required',
        },
        's': {
            'U': 'Unchanged',
            'C': 'Changed',
        },
        'c': impact_dict,
        'i': impact_dict,
        'a': impact_dict,
    }



from ..cvssv2 import calculator as v2_calculator
from ..cvssv3 import calculator as v3_calculator


def split_vector_string(vector):
    metric_rating_dict = {}
    for vector in vector.split('/'):
        metric, rating = vector.split(':')
        metric_rating_dict[metric] = rating
    return metric_rating_dict


def cvss_v2_vector_to_jira_table(vector, score):
    base_vector_string_dict = v2_calculator.get_base_vector_string_dict()
    severity_desc = v2_calculator.get_severity_description(score)

    format_items = {}
    for k, v in split_vector_string(vector).items():
        item = base_vector_string_dict[k.lower()][v]
        format_items[k] = item

    result = """
Proposed CVSS score: {0} => *{1}* severity

*Exploitability Metrics*

|| AccessVector | {AV} |
|| AccessComplexity | {AC} |
|| Authentication | {Au} |


*Impact Metrics*

|| ConfImpact | {C} |
|| IntegImpact | {I} |
|| AvailImpact | {A} |

""".format(score, severity_desc, **format_items)
    return result


def cvss_v3_vector_to_jira_table(vector, score):
    base_vector_string_dict = v3_calculator.get_base_vector_string_dict()
    severity_desc = v3_calculator.get_severity_description(score)

    format_items = {}
    for k, v in split_vector_string(vector).items():
        item = base_vector_string_dict[k.lower()][v]
        format_items[k] = item

    result = """
Proposed CVSS v3 score: {0} => *{1}* severity

*Exploitability Metrics*

|| Attack Vector | {AV} |
|| Attack Complexity | {AC} |
|| Privileges Required | {PR} |
|| User Interaction | {UI} |

*Scope Metric*

|| Scope | {S} |

*Impact Metrics*

|| Confidentiality | {C} |
|| Integrity | {I} |
|| Availability | {A} |

""".format(score, severity_desc, **format_items)
    return result



from __future__ import division
import collections


def get_impact_f_val(impact):
    if impact != 0:
        return 1.176
    return 0


def calc_base_score(impact, exploitability):
    return (((0.6 * impact) + (0.4 * exploitability) - 1.5) *
            get_impact_f_val(impact))


def get_access_complexity_score(text):
    text = text.lower()
    if text == "high":
        return 0.35
    if text == "medium":
        return 0.61
    if text == "low":
        return 0.71
    return None


def get_authentication_score(text):
    text = text.lower()
    if "none" in text:
        return 0.704
    if "single" in text:
        return 0.56
    if "multiple" in text:
        return 0.45
    return None


def get_access_vector_score(text):
    text = text.lower()
    val_map = {'local': 0.395,
               'adjacent network': 0.646,
               'network': 1}
    return val_map[text]


def _get_base_impact_val_map():
    return {'none': 0,
            'partial': 0.275,
            'complete': 0.660}


def _get_impact_score(text):
    val_map = _get_base_impact_val_map()
    return val_map[text.lower()]


def get_conf_impact(text):
    return _get_impact_score(text)


def get_integ_impact(text):
    return _get_impact_score(text)


def get_avail_impact(text):
    return _get_impact_score(text)


def get_impact_score(conf, integ, avail):
    c = _get_impact_score(conf)
    i = _get_impact_score(integ)
    a = _get_impact_score(avail)
    ret = 10.41 * (1 - ((1 - c) * (1 - i) * (1 - a)))
    return ret


def get_exploitability_score(access_v, access_comp, authen):
    av = get_access_vector_score(access_v)
    ac = get_access_complexity_score(access_comp)
    au = get_authentication_score(authen)
    return 20 * ac * au * av


def get_severity_description(score):
    """ returns the 'description' for the cvss score level provided. """
    for severity, range in get_severity_description_rating_dict().items():
        if range.bottom <= score <= range.top:
            return severity
    raise ValueError('Invalid cvss score value')


def get_severity_description_rating_dict():
    """ Returns the qualitative severity rating ranges for
        cvss v2 in a dictionary.
    """
    Range = collections.namedtuple('Range', ['bottom', 'top'])
    return collections.OrderedDict([
        ('Low', Range(0.0, 2.9)),
        ('Medium', Range(3.0, 5.9)),
        ('High', Range(6.0, 7.9)),
        ('Critical', Range(8.0, 10.0)),
    ])


def get_base_vector_string_dict():
    """ returns the base vector string dictionary. """
    impact_dict = {
        'N': 'None',
        'P': 'Partial',
        'C': 'Complete',
    }
    return {
        'av': {
            'L': 'Local',
            'A': 'Adjacent Network',
            'N': 'Network',
        },
        'ac': {
            'H': 'High',
            'M': 'Medium',
            'L': 'Low',
        },
        'au': {
            'M': 'Multiple',
            'S': 'Single',
            'N': 'None',
        },
        'c': impact_dict,
        'i': impact_dict,
        'a': impact_dict,
    }
